/*  Exile - IRC to Relational Database Gateway
 *  Copyright (C) 2002 Lucas Nussbaum <lucas@lucas-nussbaum.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: db.h 2215 2008-08-04 23:48:13Z sirvulcan $
 */

/* Exile database interface */

#ifndef _DB_H_
#define _DB_H_

#include "exile.h"
#include "log.h"

void db_connect();
int db_query(const char *fmt, ...);
char *db_escape(char *);
int db_checkserver(char *serv);
int db_getserver(char *serv);
int db_getservfromnick(char *nick);
int db_checknick(char *nick);
int db_checknumeric(char *nick);
int db_getnick(char *nick);
int db_getnumeric(char *numeric);
int db_getnick_unsure(char *nick);
int db_getnumeric_unsure(char *numeric);
int db_insertid();
void db_chgnick(char *newnick, char *oldnick);
void db_addserver(char *server, int servid);
void db_delserver(char *server);
void db_addnick(char *nick, int nickid);
void db_delnick(char *nick);
void db_addnumeric(char *numeric, int nickid);
void db_delnumeric(char *numeric);
int db_checkchannel(char *chan);
int db_getchannel(char *chan);
int db_getchancreate(char *chan);
void db_removefromchans(int nickid);
void db_removenick(char *nick);
void db_removenickid(int nickid);
void db_checkemptychan(int chanid, char *chan);
void db_close();
void db_cleanserver();
void db_cleanuser();
void db_updatehash(char *new, char *old, int nickid);
int db_getlusers(int type);
void commandcheck(char *commtype, char **av, int ac, char *source);
#define LUSERS_USERS 1
#define LUSERS_USERSINV 2
#define LUSERS_OPERS 3
#define LUSERS_SERV 4
#define LUSERS_CHAN 5
#define LUSERS_USERSGLOB 6
#define LUSERS_USERSMAX 7

#ifndef TBL_PREFIX
#define TBL_PREFIX "exile_"
#endif

	/* Tables names */
#if defined(IRCD_NEFARIOUS4)||defined(IRCD_NEFARIOUS10)
#define TBL_DNSBL TBL_PREFIX"dnsbl"
#define TBL_EXEMPT TBL_PREFIX"exempt"
#define TBL_CHEXEMPT TBL_PREFIX"chexempt"
#endif
#if defined(IRCD_NEFARIOUS10)
#define TBL_DNSBL_DATA TBL_PREFIX"dnsbl_data"
#define TBL_SHUN TBL_PREFIX"shun"
#endif

#define TBL_COMMANDS TBL_PREFIX"commands"
#define TBL_COMMQUEUE TBL_PREFIX"commandqueue"
#define TBL_COMMREPLY TBL_PREFIX"commandreply"
#define TBL_CHANLOG TBL_PREFIX"chanlog"
#define TBL_CHBAN TBL_PREFIX"chban"
#define TBL_USER TBL_PREFIX"user"
#define TBL_CHAN TBL_PREFIX"chan"
#define TBL_GLINE TBL_PREFIX"gline"
#define TBL_ZLINE TBL_PREFIX"zline"
#define TBL_ISON TBL_PREFIX"ison"
#define TBL_SERV TBL_PREFIX"server"
#define TBL_MAXV TBL_PREFIX"maxvalues"
#define TBL_GEOIP TBL_PREFIX"geoip"
#define TBL_STATS_l TBL_PREFIX"stats_l"
#define TBL_SILENCE TBL_PREFIX"silence"
#define TBL_VERSION TBL_PREFIX"version"

#endif
