/*  Exile - IRC to Relational Database Gateway
 *  Copyright (C) 2006 Neil Spierling <sirvulcan@gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: geoip.c 1775 2006-09-10 16:47:14Z sirvulcan $
 */

#include "config.h"
#include "exile.h"
#include "db.h"
#include "misc.h"
#include "memory.h"
#include "log.h"
#include "actions.h"
#include "sockutil.h"

#if defined(HAVE_LIBGEOIP)&&defined(HAVE_GEOIP_H)&&defined(HAVE_GEOIPCITY_H)
#include <GeoIP.h>
#include <GeoIPCity.h>

GeoIP * gi = NULL;
GeoIP * cgi = NULL;

extern char *GeoIPDataFile;
extern char *GeoIPCityFile;
#endif

extern int UserCacheTime;
MYSQL *myptr;

void
unset_geoip_info(int nickid)
{
    if (UserCacheTime)
        db_query("DELETE FROM " TBL_GEOIP " WHERE nickid='%d'", nickid);
    return;
}

void
unset_geoip_info_cleanup(int nickid)
{
    db_query("DELETE FROM " TBL_GEOIP " WHERE nickid='%d'", nickid);
    return;
}

int check_nickid(int nickid)
{
    MYSQL_RES *resptr;
    int rid = 0;
    char **res;

    db_query("SELECT nickid FROM " TBL_GEOIP " WHERE nickid='%d'", nickid);

    resptr = mysql_store_result(myptr);
    while ((res = mysql_fetch_row(resptr)))
        rid = atoi(res[0]);

    return rid;
}

void
set_geoip_info(int nickid, char *host)
{
/* Need the libs and the headers if this is going to compile properly */
#if defined(HAVE_LIBGEOIP)&&defined(HAVE_GEOIP_H)&&defined(HAVE_GEOIPCITY_H)
    GeoIPRecord * gir;

    if (!GeoIPDataFile && !GeoIPCityFile)
        return; /* Admin doesnt want to use geoip functions */

    if (GeoIPDataFile)
        if (!gi)
            gi  = GeoIP_open(GeoIPDataFile, GEOIP_MEMORY_CACHE | GEOIP_CHECK_CACHE);

    if (GeoIPCityFile)
        if (!cgi)
            cgi = GeoIP_open(GeoIPCityFile, GEOIP_MEMORY_CACHE | GEOIP_CHECK_CACHE);

    if (cgi) {
        gir = GeoIP_record_by_name(cgi, host);
        if (gir) {
            if (UserCacheTime && (check_nickid(nickid) != 0)) {
		db_query("UPDATE " TBL_GEOIP " SET country_name='%s', country_code='%s', city='%s', region='%s', postal_code='%s', longitude='%f', latitude='%f', dma_code='%d', area_code='%d' WHERE nickid='%d'",
			gir->country_name ? db_escape(gir->country_name) : "",
			gir->country_code ? db_escape(gir->country_code) : "",
			gir->city ? db_escape(gir->city) : "",
			gir->region ? db_escape(gir->region) : "",
			gir->postal_code ? gir->postal_code : "",
			gir->longitude ? gir->longitude : 0,
			gir->latitude ? gir->latitude : 0, 
			gir->dma_code ? gir->dma_code : 0,
			gir->area_code ? gir->area_code : 0,
			nickid);
            } else {
		db_query
			("INSERT INTO " TBL_GEOIP " (nickid, country_name, country_code, city, region, postal_code, longitude, latitude, dma_code, area_code) VALUES('%d', '%s', '%s', '%s', '%s', '%s', '%f', '%f', '%d', '%d')",
			nickid, gir->country_name ? db_escape(gir->country_name) : "",
				gir->country_code ? db_escape(gir->country_code) : "",
				gir->city ? db_escape(gir->city) : "",
				gir->region ? db_escape(gir->region) : "",
				gir->postal_code ? gir->postal_code : "",
				gir->longitude ? gir->longitude : 0,
				gir->latitude ? gir->latitude : 0, 
				gir->dma_code ? gir->dma_code : 0,
				gir->area_code ? gir->area_code : 0);
            }
            GeoIPRecord_delete(gir);
            return;
	}
    } else if (gi) {
        const char *country = GeoIP_country_name_by_name(gi, host);
        if (country) {
            if (UserCacheTime && (check_nickid(nickid) != 0))
		db_query("UPDATE " TBL_GEOIP " SET country_name='%s' WHERE nickid='%d'", country, nickid);
            else
		db_query("INSERT INTO " TBL_GEOIP " (nickid, country_name) VALUES('%d', '%s')", nickid, country);
        }
        return;
    }

    return;
#endif
}
